header
{
	using System.IO;
}
options
{
	language = "CSharp";
	namespace = "Agilent.Vee.Addin.ExpressionGraph";
}

class ExpressionParser extends Parser;
options
{
	exportVocab = ExpressionGraph;
	buildAST = true;
}
imaginaryTokenDefinitions :
   SIGN_MINUS
   SIGN_PLUS
;

expr : LPAREN^ sumExpr RPAREN!;

sumExpr  : mulExpr ( ( PLUS^ | MINUS^ ) mulExpr )*;

mulExpr  : powExpr ( ( MUL^ | DIV^ ) powExpr )*;

powExpr  : ( signExpr | funcExpr ) ( POW^ ( signExpr | funcExpr ) )?;

signExpr : ( m:MINUS^ {#m.setType(SIGN_MINUS);} |
             p:PLUS^  {#p.setType(SIGN_PLUS);} )? atom ;

funcExpr : ( ABS^ | ACOS^ | ASIN^ | ATAN^ | COS^ | EXP^ | LOG^ | LOG10^ | SQRT^ | SIN^ | TAN^ ) expr;

atom : NUMBER | X | expr;

class ExpressionLexer extends Lexer;
options
{
	caseSensitive = false;
	k = 4;
	exportVocab = ExpressionGraph;
}
WS : ( ' ' |	'\t' |	'\n' |	'\r' )+
{
	$setType(Token.SKIP);
};

NUMBER : ('0'..'9') + ('.' ('0'..'9')*)?;
X      : "x";

ABS    : "abs";
ACOS   : "acos";
ASIN   : "asin";
ATAN   : "atan";
COS    : "cos";
EXP    : "exp";
LOG    : "log";
LOG10  : "log10";
SQRT   : "sqrt";
SIN    : "sin";
TAN    : "tan";

PLUS   : '+';
MINUS  : '-';
MUL    : '*';
DIV    : '/';
POW    : '^';

LPAREN : '(';
RPAREN : ')';


class ExpressionTreeParser extends TreeParser;
options
{
	exportVocab = ExpressionGraph;
}
{
	private AST _ast;
	
	public ExpressionTreeParser(string expression) : this()
	{
		ExpressionLexer lexer = new ExpressionLexer(new StringReader(expression));
		ExpressionParser parser = new ExpressionParser(lexer);
		parser.expr();
		_ast = parser.getAST();
	}
	
	public double XValue { get; set; }
	
	public double Value
	{
		get
		{
	  		double y = eval(_ast);
	  		return y;
		}
	}
}
eval returns [double v = 0]
 : v=expr
;

protected expr returns [double v = 0]
{
	double l = 0;
	double r = 0;
}
 : i:NUMBER                      { v = Convert.ToDouble(i.getText()); }
 | x:X		                     { v = XValue; }

 | #( ABS        l=expr)         { v = Math.Abs(l);}
 | #( ACOS       l=expr)         { v = Math.Acos(l);}
 | #( ASIN       l=expr)         { v = Math.Asin(l);}
 | #( ATAN       l=expr)         { v = Math.Atan(l);}
 | #( COS        l=expr)         { v = Math.Cos(l);}
 | #( EXP        l=expr)         { v = Math.Exp(l);}
 | #( LOG        l=expr)         { v = Math.Log(l);}
 | #( LOG10      l=expr)         { v = Math.Log10(l);}
 | #( SQRT       l=expr)         { v = Math.Sqrt(l);}
 | #( SIN        l=expr)         { v = Math.Sin(l);}
 | #( TAN        l=expr)         { v = Math.Tan(l);}

 | #( PLUS       l=expr r=expr ) { v = l + r; }
 | #( MINUS      l=expr r=expr ) { v = l + r; }
 | #( DIV        l=expr r=expr ) { v = l / r; }
 | #( MUL        l=expr r=expr ) { v = l * r; }
 | #( POW        l=expr r=expr ) { v = Math.Pow(l, r); }

 | #( SIGN_MINUS l=expr)         { v = 0 - l; } 
 | #( SIGN_PLUS  l=expr)         { v = l; } 
 | #( LPAREN     l=expr)         { v = l;}
 ;
